require "ruby-debug"
Debugger.start

require "rubygame"
require "rubygame/tools"
require "opengl"
#require "rubygame/opengl_tools"
include Rubygame
include Rubygame::GL
include GL
include GLU

WIDTH = 600.0
HEIGHT = 600.0

$screen = Screen.set_mode [WIDTH, HEIGHT], 32, OPENGL

register :key => K_ESCAPE do throw :quit end

Viewport 0, 0, WIDTH,HEIGHT

MatrixMode PROJECTION
LoadIdentity()
Perspective 32, WIDTH/HEIGHT, 3, 30
Translate 0,0,-7.5

MatrixMode MODELVIEW
LoadIdentity()

Enable DEPTH_TEST
DepthFunc LESS
ShadeModel FLAT

cube = [[ 1.0, 1.0, -1.0], 
        [ 1.0, -1.0, -1.0],
        [ -1.0, -1.0, -1.0],
        [ -1.0, 1.0, -1.0],
        [ 1.0, 1.0, 1.0],
        [ 1.0, -1.0, 1.0],
        [ -1.0, -1.0, 1.0],
        [ -1.0, 1.0, 1.0]]

#$outline_cube = List.new do 
NewList 1, COMPILE_AND_EXECUTE
  PushMatrix()
  Begin LINE_STRIP
    Color 1,1,1
    Vertex cube[0]
    Vertex cube[1]
    Vertex cube[2]
    Vertex cube[3]
    Vertex cube[0]
    Vertex cube[4]
    Vertex cube[5]
    Vertex cube[6]
    Vertex cube[7]
    Vertex cube[4]
  End()
  Begin LINES
    Vertex cube[1]
    Vertex cube[5]

    Vertex cube[2]
    Vertex cube[6]

    Vertex cube[3]
    Vertex cube[7]
  End()
  PopMatrix()
EndList()

#$filled_cube = List.new do
NewList 2, COMPILE_AND_EXECUTE
  Begin QUADS
    [0,1,2,3].each{|i| Vertex cube[i]}
    [0,4,5,1].each{|i| Vertex cube[i]}
    [4,5,6,7].each{|i| Vertex cube[i]}
    [7,6,2,1].each{|i| Vertex cube[i]}
    [0,1,7,4].each{|i| Vertex cube[i]}
    [1,2,6,5].each{|i| Vertex cube[i]}
  End()
EndList()

class Field
  def initialize
    @angle, @height, @zoom = 10.0, -12.0, 0.7
  end
  def rotate a,h
    @angle += a
    @height += h
  end
  def zoom z
    @zoom += z/20.0
  end
  def draw
    LoadIdentity()
    Rotate -@angle, 0, 1, 0
    Rotate -@height, 1, 0, 0
    Scale @zoom,@zoom,3*@zoom
    CallList 1
  end
end
field = Field.new

class Pad
  attr_reader :x, :y
  def initialize z = 1
    @x, @y, @z = 0, 0, z
  end
  def draw
    PushMatrix()
    Translate @x, @y, @z
    Scale 0.2,0.2,0.01
    CallList 1
    PopMatrix()
  end
  def move_to x,y
    @x = x<-0.8 ? -0.8 : (x>0.8 ? 0.8 : x)
    @y = y<-0.8 ? -0.8 : (y>0.8 ? 0.8 : y)
  end
  def within x,y
    @x-0.2 < x and @x+0.2 > x and @y-0.2 < y and @y+0.2 > y
  end
end
pad = Pad.new
other_pad = Pad.new -1

class Ball
  def initialize
    @pos = [0,0,0]
    @rot = [0,0,0]
    @vel = [0.013,0.02,0.01]
  end

  def draw
    PushMatrix()
    Translate @pos[0],@pos[1],@pos[2]
    Scale 0.025,0.025,0.025/3
    Rotate @rot[0], 1,0,0
    Rotate @rot[1], 0,1,0
    Rotate @rot[2], 0,0,1
    CallList 2
    PopMatrix()
  end
  
  def move
    for i in 0..2 do
      @pos[i] += @vel[i]
      @vel[i] *= -1 if @pos[i] < -1 or @pos[i] > 1
      @rot[i] += 9
    end
  end
  
  def score pad, other_pad
    return -1 if @pos[2] <= -1 and not other_pad.within @pos[0], @pos[1]
    return 1 if @pos[2] >= 1 and not pad.within @pos[0], @pos[1]
  end
end

ball = Ball.new

old_pos = nil
register :mouse_motion do |e|
  if $left_button
    field.rotate old_pos[0]-e.pos[0], old_pos[1]-e.pos[1]
  else
    pad.move_to( ((e.pos[0]-WIDTH/4)*4-WIDTH)/WIDTH, -((e.pos[1]-HEIGHT/4)*4-HEIGHT)/HEIGHT )
  end
  old_pos = e.pos
end

$key = {}
register :button => 1, :class => [MouseDownEvent, MouseUpEvent] do |e|
  $left_button = e.is_a? MouseDownEvent
end
register :button => 4 do field.zoom 1 end
register :button => 5 do field.zoom -1 end
register :key => K_A do other_pad.move_to other_pad.x-0.1, other_pad.y end
register :key => K_W do other_pad.move_to other_pad.x, other_pad.y+0.1 end
register :key => K_D do other_pad.move_to other_pad.x+0.1, other_pad.y end
register :key => K_S do other_pad.move_to other_pad.x, other_pad.y-0.1 end

register :key => /^[awsd]$/, :class => [KeyDownEvent, KeyUpEvent] do |e|
  $key[e.key] = e.is_a?(KeyDownEvent)
end


$next_keypress_lifetime = 0
register :tick do
  ball.move
  
  ClearColor 0,0,0,1
  Clear COLOR_BUFFER_BIT | DEPTH_BUFFER_BIT

  MatrixMode MODELVIEW
  field.draw
  pad.draw
  other_pad.draw
  ball.draw

  if $clock.lifetime > $next_keypress_lifetime
    [K_A, K_W, K_D, K_S].each do |k|
      $queue.post KeyDownEvent.new(k,[]) if $key[k]
    end
    $next_keypress_lifetime = $clock.lifetime + 50
  end

  ball = Ball.new if ball.score pad, other_pad

  swap_buffers
end

start
